---
title: 计算之魂
description: 计算科学品位和认知进阶
date: 2021-12-09
---

- [计算之魂](https://book.douban.com/subject/35641088/)
  - 副标题: 计算科学品位和认知进阶
  - 抖音上面买的~
  - 吴军老师的书, 一向很具条理性

## 计算的本质: 从机械到电子

- 希尔伯特一直在思考这样三个问题.
  - 数学是完备的 (complete) 吗?
  - 所谓`完备性`, 就是说对于任意一个命题,
    要么可以证明它是对的, 要么可以证明它是错的.
  - 数学是一致的 (consistent)吗?
  - 所谓`一致性`, 就是说一个命题不能既是真的, 又是假的.
  - 数学是可判定的 (decisive) 吗?
  - 所谓`可判定性`, 就是说一个具体的问题,
    你能否判断它是否有答案.
- 希尔伯特的这三个问题从本质上划定了数学的边界.
  因为数学只能解决那些在数学上是完备的问题,
  而数学的一致性则能保证它没有似是而非的答案.
- 后来著名数学家哥德尔解决了前两个问题,
  他指出`数学不可能既是完备的, 又是一致的`,
  这就是数学中著名的哥德尔不完全性定理.
- 哥德尔的理论告诉我们, 世界上有很多问题我们无法判定它们的对错,
  因此它们不是数学问题.

## 毫厘千里之差: 大 O 概念

- 高德纳的思想主要包括以下三个部分:
  - 在比较算法的快慢时, 只需要考虑数据量特别大,
    大到近乎无穷大时的情况.
  - 决定算法快慢的因素虽然可能有很多,
    但是所有的因素都可以被分为两类.
    第一类是不随数据量变化的因素,
    第二类是随数据量变化的因素.
  - 两种算法在复杂度上相差哪怕只有一点点,
    `N` 很大之后, 效率可能就差出万亿倍了.

- 时间顺序
  - `1945`, 归并排序. 计算复杂度 `O(N logN)`, 额外空间 `O(N)`
  - `1964`, 堆排序, 就地排序.
  - 快速排序, 额外空间 `O(logN)`, 计算复杂度常数级优化 (`2~3` 倍)

- 归并排序中还有第二个问题要回答, 那就是上述归并的过程要进行多少次.
  - 我们知道当子序列中只剩下一个元素时,
    上述逐级往下递归的过程就结束了,
  - 而每一次递归循环, 都会将当前序列中的元素数量减半,
    那么只要 `logN` 次递归, 就遇到了结束条件.
- 每一次递归, 从上到下, 所需要的计算量分析如下:
  - 第一次, `O(N)`.
  - 第二次, 由于有两个子序列, 而每个子序列的计算量是原先的一半,
    因此总的计算量和第一次相同, 我们可以认为是 `O(N/2*2) = O(N)`.
  - 第三次, 有了四个子序列, 每一个是原始序列的 `1/4`,
    因此总的计算量和第一次相同.
  - 最后, 到了第 `K = logN` 次, 有了 `N` 个子序列,
    每一个是原始序列的 `1/N`, 总计算量依然等于第一次的 `N`.
- 这也就是说, 整个归并排序的计算复杂度是 `O(N logN)`.

- `蒂姆排序`这个名字来源于该算法的发明人`蒂姆·彼得斯` (Tim Peters),
  他在 `2002` 年发明了一种将两种排序算法的特点相结合,
  - 插入排序节省内存,
  - 归并排序节省时间,
- 最坏时间复杂度控制在 `O(N logN)` 量级,
  同时还能够保证排序`稳定性`这样一举三得的混合排序算法.
  - `蒂姆排序`最初是在 `Python` 程序语言中实现的,
    今天它依然是这种程序语言默认的排序算法.

- 常见的几种高效排序算法的对比

|    算法    | 平均时间复杂度 | 最坏时间复杂度 | 额外空间复杂度 | 稳定性 |
| --------- |:------------:|:------------:|:------------:|:----:|
|  归并排序   |  O(N logN)  |   O(N logN)  |     O(N)     |  Yes  |
|  堆排序    |   O(N logN)  |   O(N logN)  |     O(1)     |  NO   |
|  快速排序   |  O(N logN)  |   O(N^2)     |    O(logN)   |  NO   |

## 智能的本质: 分类与组合

### 十条集合公理

- 外延公理 (Axiom of Extensionality):
  - 一个集合完全由它的元素所决定.
  - 两个集合相同, 当且仅当它们的元素皆相同.
- 分类公理 (子集公理)
  - (Axiom Schema of Specification)
  - (Axiom Schema of Separation)
  - (Axiom Schema of Restricted Comprehension):
  - 给出任何集合 `A` 及命题 `P(x)`, 其中
    $$ x \in A $$,
    存在着一个原来集合的子集 `B` (即
    $$ B \subseteq A $$
    ), 包含且只包含使 `P(x)` 成立的元素.
- 配对公理 (Axiom of Pairing):
  - 假如 `X` 和 `Y` 均为集合, 则有另一个集合
    $$ \{ X, Y \} $$
    包含 `X` 与 `Y` 作为它仅有的元素.
- 并集公理 (Axiom of Union):
  - 对于任意一个集合 `X`, 存在一个并集 `Y`,
    `Y` 的元素是且只会是 `X` 的元素的元素.
- 空集公理:
  - 存在着一个不包含任何元素的集合, 我们将这个空集合记为
    $$ \varnothing $$.
  - 这一条公理可以由分类公理得出.
- 无穷公理 (Axiom of Infinity):
  - 存在着一个集合 `X`, 空集
    $$ \varnothing $$
    为其元素之一且对于任何 `X` 中的元素 `Y`,
    $$ Y \cup \{ Y \} $$
    也是 `X` 的元素.
  - 说明: 用这种方式, 可以定义所有的整数.
- 替代公理 (Axiom Schema of Replacement):
  - 任给一个集合 `A` 和映射关系 (泛函谓词) `F`, 存在一个集合 `B`,
    `B` 的所有元素是集合元素在 `F` 映射下的像.
- 幂集公理 (Axiom of Power Set):
  - 每一个集合都有其幂集, 即对于任何集合 `X` 存在着一个集合 `Y`,
    使得 `Y` 的元素是且只会是 `X` 的子集.
- 正则公理
  - (Axiom of Regularity/Axiom of Foundation):
  - 每一个非空集合 `X` 总包含着一个元素 `y`,
    使得 `X` 与 `y` 无交集.
  - 说明: 这条公理是为了防止出现罗素悖论.
- 选择公理 (Axiom of Choice, Zermelo's Aersion):
  - 任给一个集合 `X`, 其元素为互不相交的非空集,
    对于 `X` 中的任意一个元素 `Y`, 总存在着一个函数
    `f` (称为 `Y` 的一个选择函数), 使得
    $$ f(Y) \in Y $$.

### 卡特兰数

- 有多少种 `N` 个叶子结点的满二叉树 (Full Binary Tree)?

## 工具与算法: 图论及应用

### 动态规划 (Dijkstra 算法)

## 化繁为简: 分治思想及应用

### 分割算法

## 状态与流程: 等价性与因果关系

- 但是处理器内部并没有除法器,
  实际上除法是通过`乘法`, `加减法`和`部分查表`来完成的,
  一次除法相当于`十几次`的`乘法`和`加减法`.
- 这也就是说,
  除法可以通过`十几步`操作大致等价为一系列处理器现有功能能够实现的操作.
- 当然, 了解了除法实现的这个细节,
  大家在写代码时要尽可能多用乘法,
  少用除法, 从而提高运算速度.

## 确定与随机: 概率算法及应用

## 理论与实战: 典型难题精解
