---
title: TAOCP 半数值算法 - 计算机程序设计艺术
description: 穷且益坚, 不坠青云之志. 酌贪泉而觉爽, 处涸辙以犹欢.
date: 2021-06-26
---

- [Wiki - The Art of Computer Programming](https://en.wikipedia.org/wiki/The_Art_of_Computer_Programming)

- [计算机程序设计艺术](https://book.douban.com/series/46236)
  - [计算机程序设计艺术 卷2](https://book.douban.com/subject/26850558/)

- 习题答案占据了`1/4`左右的篇幅

------------------

## 随机数

> 随机数不应该用随机选择的方法生成, 应该使用某种理论

- 线性同余法
  - 模: `m` (`m > 0`)
  - 乘数: `a` (`0 <= a < m`)
  - 增量: `c` (`0 <= c < m`)
  - 初值: `X_0` (`0 <= X_0 < m`)
  - $$ X_{n + 1} = (a \times X_{n} + c) \mod m $$
    (`n >= 0`)

- 由 `m`, `a`, `c` 和 `X_0` 定义的线性同余序列具有周期 `m`, 当且仅当
  1. `c` 与 `m` 互素;
  2. 对于每个整除 `m` 的素数 `p`, `b = a - 1` 是 `p` 的倍数;
  3. 如果 `m` 是 `4` 的倍数, 则 `b` 是 `4` 的倍数.

- 如果
  $$ m = 10^e $$,
  `e >= 5`, `c = 0`, 并且 `X_0` 不是 `2` 或 `5` 的倍数,
  则线性同余序列的周期为
  $$ 5 \times 10^{e-2} $$,
  当且仅当 `a mod 200` 等于如下 `32` 个值之一:
  - `3`, `11`, `13`, `19`, `21`, `27`, `29`, `37`,
  - `53`, `59`, `61`, `67`, `69`, `77`, `83`, `91`,
  - `109`, `117`, `123`, `131`, `133`, `139`, `141`, `147`,
  - `163`, `171`, `173`, `179`, `181`, `187`, `189`, `197`

- `势`
  - 对于随机性而言, 高势是必要但不充分的

### 统计检验

------------------

## 算术

- 按位记数系统
  - 二进制, 莱布尼兹

- 复基数记数系统
  - 基数为 `2i` 的记数系统称为`虚四进制`数系

- 平衡三进制数系, `-1`, `0`, `1`, 三元
  - 柯西

- 混合进制数系
  - 阶乘数系

### 浮点算术

- 单精度计算
  - 加法, 减法, 规范化

- 双精度计算

- 多精度计算

### 最大公因数

- __现代欧几里得算法__
  - 给定两个`非负`整数 `A` 和 `C`, 这个算法求它们的最大公因数.
    (要求任意两个整数 `u` 和 `v` 的最大公因数, 只需将这个算法用于 `|u|` 和 `|v|`.)
  - (1) `[ v = 0 ? ]` 如果 `v = 0`, 则算法结束, 而 `u` 就是答案
  - (2) `[ 取 u mod v. ]` 置 `r <- u mod v, u <- v, v <- r`, 然后回到 `1`.
    (这一步运算可以使 `v` 的值变小, 同时不改变 `gcd(u, v)`.)
