---
title: 基础拓扑学
description: 江流宛转绕芳甸, 月照花林皆似霰; 空里流霜不觉飞, 汀上白沙看不见.
date: 2021-08-24
---

- [基础拓扑学 (修订版)](https://book.douban.com/subject/34899142/)
  - 原作名: __Basic Topology__

> 本书并未提前详述一些分析的概念, 所以需要阅读前置书籍, 比如:
  [普林斯顿数学分析读本](https://book.douban.com/subject/35172355/)

```
常见变换的性质比较

变换名称 自由度 不变性质
欧氏变换  6    长度, 夹角, 体积
相似变换  7    体积比
仿射变换  12   平行性, 体积比
射影变换  15   接触平面的相交和相切
```

- 一个`多面体`是指按下述意义很好地拼凑在一起的有限多个`平面多边形`:
  - 若两个多边形相交, 则它们交于一条公共边;
  - 多边形的每一条边恰好还是另一个 (且只有这一个) 多边形的边.
  - 不仅如此, 还要求对于每个顶点, 那些含有它的多边形可以排列成
    $$ Q_1, Q_2, ..., Q_k $$,
    使得
    $$ Q_i $$
    与
    $$ Q_{ i + 1 } $$
    有一条公共边,
    $$ 1 ≤ i < k $$,
    而
    $$ Q_k $$
    与
    $$ Q_1 $$
    有一条公共边.
  - 换句话说, 这些多边形拼成围绕着该顶点的一块区域
    (多边形的数目 `k` 则可以随顶点的不同而变动).
- 其中最后一个条件就使两个立方体只在一个公共顶点相衔接的情形排除在外了.

---

- __Euler 定理__ 设 `P` 为满足下列条件的多面体:
  - (1) `P` 的任何两个顶点可以用一串棱相连接;
  - (2) `P` 上任何由直线段(不一定非是 `P` 的棱)构成的圈,
    把 `P` 分割成两片.
  - 则对于 `P` 来说, `v - e + f = 2`.
  - 若以 `T` 来记`树`形, 则可以写成公式 `v(T) - e(T) = 1`

- 拓扑等价的多面体具有相同的 `Euler` 数.

```
空间的每点有一组"邻域", 这些邻域又引出了连续映射的适当定义, 这就是关键所在.
注意在欧氏空间内定义邻域时完全依靠两点之间的欧氏距离, 在构造抽象空间时,
我们希望保留邻域的概念, 但要避免对距离概念的任何依赖 (拓扑等价不保持距离).
```

- [拓扑空间](https://en.wikipedia.org/wiki/Topological_space)
  - __邻域__

- __定义__ 曲面是这样的拓扑空间,
  - 它的每一点有同胚于平面的邻域,
  - 并且任意不同的两点有不相交的邻域.

- __分类定理__ 任何闭曲面或者同胚于球面,
  或者同胚于添加了有限多个环柄的球面,
  或者同胚于挖去有限多个圆盘而以莫比乌斯带代替的球面.
  - 而这些曲面中的任何两个都不同胚.

- [基本群](https://en.wikipedia.org/wiki/Fundamental_group)

### 连续性

- __定义__ 集合 __X__ 上的一个`拓扑`是由 __X__ 的子集所构成的一个非空组,
  它的成员叫作开集, 它们满足下列要求:
  - 任意多个开集的并集是开集,
  - 有限多个开集的交集是开集,
  - __X__ 与空集是开集.
- 集合配备了它上面的一个拓扑以后叫作`拓扑空间`.
  - 以后, 我们将采用这个定义.

- [度量空间](https://en.wikipedia.org/wiki/Metric_space)

- __Tietze 扩张定理__
  在度量空间的闭子集上定义的任何连续实值函数可以扩张到整个空间.

### 紧致性与连通性

- 连通性的定义可以用多种方式来陈述. __定理__
  对于空间 __X__, 下列条件是等价的:
  - (a) __X__ 连通;
  - (b) __X__ 内同时为开集与闭集的子集只有 __X__ 与空集;
  - (c) __X__ 不能表示为两个不相交非空开集的并集;
  - (d) 不存在连续满映射从 __X__ 到一个多于一点的离散拓扑空间上去.

### 单纯剖分

```
把全体拓扑空间作为我们讨论的对象未免过于宽泛.
在前几章我们曾经看到怎样建立拓扑空间与连续映射的抽象理论, 并且证明了许多重要结果.
但是, 在这样宽泛的基础上进行工作, 很快就会碰到两类困难.
一方面, 当人们试图证明具体的几何结果, 比如像曲面的拓扑分类时,
曲面的 (局部欧氏的) 纯拓扑结构不能提供很多从何着手的依据;
另一方面, 虽然能对很一般的拓扑空间定义代数的不变量, 诸如基本群,
但是, 除非我们能对足够广泛的一类空间对这些不变量进行计算, 它们的用处仍然显不出来.
但如果空间可由很多我们所熟识的空间很好地拼凑起来构成,
即空间是所谓可单纯剖分的空间, 这两个问题都能得到有效的处理.
```

- [单纯复形](https://en.wikipedia.org/wiki/Simplicial_complex)
  - [单纯形](https://en.wikipedia.org/wiki/Simplex)
- [自由群](https://en.wikipedia.org/wiki/Free_group)
  - [自由积](https://en.wikipedia.org/wiki/Free_product)
- [同调论](https://en.wikipedia.org/wiki/Homology_(mathematics))
- [塞弗特-范坎彭定理](https://en.wikipedia.org/wiki/Seifert-Van_Kampen_theorem)

```
这里给出无穷复形的一个尝试性的定义, 并不算最广义的, 但对我们的需要来说已经足够.
```

- __定义__ 欧氏空间
  $$ \mathbf{E}^n $$
  内一个由单纯形构成的无穷集合, 如果满足下列条件就叫作无穷单纯复形:
  - (a) 若某个单纯形属于这个集合, 则它所有的面也属于该集合;
  - (b) 这个集合的单纯形很好地拼合;
  - (c) 这个集合全体单纯形的并集是
    $$ \mathbf{E}^n $$
    内的闭集;
  - (d) 在这个并集上, 诱导拓扑与粘合拓扑重合.

- __定理__ 设 `K` 为
  $$ \mathbf{E}^n $$
  内的无穷单纯复形, 以
  $$ \mid K \mid $$
  表示它的多面体, 则
  - (a) `K` 是有限维的;
  - (b) `K` 的单纯形总数是可数的;
  - (c) `K` 是局部有限的 (就是说, `K` 的每个顶点只在有限多个单纯形上);
  - (d)
    $$ \mathbf{E}^n $$
    的每一点有那样的邻域, 它至多只与有限多个 `K` 的单纯形相交.

### 曲面

- [曲面](https://en.wikipedia.org/wiki/Surface_(topology))

- [欧拉示性数](https://en.wikipedia.org/wiki/Euler_characteristic)

- __定理__ 每个闭曲面同胚于标准闭曲面之中的一个.

```
H(p) 叫作亏格为 p 的标准可定向曲面, M(q) 叫作亏格为 q 的标准不可定向曲面.
一旦知道一个闭曲面的亏格以及是否可定向, 这个闭曲面就完全决定了.
```

---

- 近来拓扑学方面的许多工作围绕着关于流形的研究. 这是类似于曲面的高维对象.
  一个 `n` 维流形, 或简称 `n` 流形, 是一个第二可数的 Hausdorff 空间,
  它的每一点有邻域同胚于
  $$ \mathbf{E}^n $$.
- 空间
  $$ \mathbf{E}^n $$,
  $$ S^n $$,
  $$ P^n $$
  都是 `n` 流形;
  $$ S^3 \times S^1 $$
  是闭的 `4` 流形 ("闭" 意思是紧致连通);
  `n` 流形内的任何开集是 `n` 流形, 因此,
  $$ GL(n) $$
  是
  $$ n^2 $$
  维流形;
  $$ SO(n) $$
  是一个
  $$ n (n-1) / 2 $$
  维闭流形; 最后, 透镜空间
  $$ L(p, q) $$
  都是闭 `3` 流形.
  - 这方面的研究虽然取得了很大进展, 但许多基本问题尚未得到解答.
  - 最重要的是著名的 `Poincaré` 猜想.
  - 如果表达成问题的形式, 它问道, 是否每一个单连通的三维闭流形同胚于
    $$ S^3 $$.

---

- [同调](https://en.wikipedia.org/wiki/Homology_(mathematics))
- [映射度](https://en.wikipedia.org/wiki/Degree_of_a_continuous_mapping)
- [Lefschetz fixed-point](https://en.wikipedia.org/wiki/Lefschetz_fixed-point_theorem)
