---
title: 量子计算与量子信息 - 计算
description: 昨夜西风凋碧树, 独上高楼, 望尽天涯路. 欲寄彩笺无尺素, 山长水阔知何处!
date: 2023-01-17
---

- [量子计算与量子信息](https://book.douban.com/subject/35777059/)

## 量子电路

### 量子算法

- [量子计数](https://en.wikipedia.org/wiki/Quantum_counting_algorithm)

### 单量子比特运算

### 受控运算

### 测量

### 通用量子门

### 量子电路模型计算总结

### 量子系统的模拟

## 量子傅里叶变换及其应用

### 量子傅里叶变换

### 相位估计

### 应用: 求阶与因子分解问题

### 量子傅里叶变换的一般应用

## 量子搜索算法

### 量子搜索算法

### 作为量子模拟的量子搜索

### 量子计数

### NP 完全问题解的加速

### 无结构数据库的量子搜索

### 搜索算法的最优性

### 黑盒算法的极限

## 量子计算机: 物理实现

> 这一章比较鸡肋~
  物理实现种类多样, 工艺复杂, 对于非从业者, 意义不大.
  且在可预见的未来存在很大变数! 估计成书之后, 已经有了些变化~
  但是, 概略性的介绍其实也是有意义的, 浏览下基础理论即可.
  所以提前略读了这一章, 做为第一部分阅读 (Part 1) 的收尾!

- 实现量子计算有 `4` 项基本要求:
  1. 量子比特表示,
  2. 可控酉变换,
  3. 初始量子比特态的制备,
  4. 最终量子比特态的测量.

### 量子计算的条件

- 量子比特是一些二能级系统, 作为量子计算机最简单的建造单元,
  它们为成对的量子态提供了方便的标志和物理实现.
  - 因此, 比如自旋 `3/2` 粒子的 `4` 个态,
    $$ \mid m = + 3/2 \rangle $$,
    $$ \mid m = + 1/2 \rangle $$,
    $$ \mid m = - 1/2 \rangle $$,
    $$ \mid m = - 3/2 \rangle $$,
    可以用来代表两个量子比特.
- 以计算作为目的, 要实现的关键是可访问态的集合应该是`有限的`.
  沿一维直线运动粒子的位置 `x` 通常不适合作为计算态的集合, 尽管粒子可能处于量子态
  $$ \mid x \rangle $$,
  乃至叠加态
  $$ \sum_{x} c_x \mid x \rangle $$.
  - 这是由于 `x` 处于概率上的连续区域, 且具有无限大小的希尔伯特空间,
    因此无噪声时其信息容量也是无限的.
  - 比如说, 在完美世界中, 莎士比亚全集可以存储到无限位的二进制小数中
    $$ x = 0.010111011001... $$
    (并读出).
  - 这显然是不现实的, 现实是噪声的存在把可分态数目降为有限个.
- 实际上, 通常需要把某些对称性献给态空间的有限性, 以便把退相干降到最小.
  - 比如说, 一个自旋 `1/2` 粒子的希尔伯特空间由
    $$ \mid \uparrow \rangle $$
    和
    $$ \mid \downarrow \rangle $$
    两个态张成;
  - 自旋态不能处于此二维空间之外, 当被很好地孤立之后,
    就成为一个近乎完美的量子比特.

### 量子计算机的实现

> 具体的量子计算机实现粗略的浏览了下~
  介绍的也比较概括.

- [量子谐振子](https://en.wikipedia.org/wiki/Quantum_harmonic_oscillator)
- [F-P 腔](https://en.wikipedia.org/wiki/Fabry-Pérot_interferometer)
- [Jaynes-Cummings model](https://en.wikipedia.org/wiki/Jaynes-Cummings_model)
- [偏振](https://en.wikipedia.org/wiki/Polarization_(waves))

- 一般来说, 对一个物理系统而言能完成一个酉变换 __U__ 的充要条件很简单,
  即为系统的时间演化算子
  $$ T = exp(-i \mathbf{H} t) $$,
  由哈密顿量 __H__ 来定义, 具有与 __U__ 近乎相同的本征值谱.

```
也许最重要的, 是腔 QED 开启了对量子信息处理意义非凡的, 通向附加相互作用宝藏的大门.
我们也看到, 如何从量子信息的视角 -- 限定于单光子与单原子 -- 使我们能用最基本的腔
QED 相互作用 Jaynes-Cummings 哈密顿量, 构建出电磁波与物质相互作用的一些最基本的物理.
```

```
当离子具有自旋时, 它拥有磁矩, 就像是一个复合粒子, 其中有一些沿着环路运动的电流.
但是电子是基本粒子, 且已知组成原子核的夸克不会通过轨道运动产生自旋.
不仅如此, 一个粒子的自旋只能是整数或半整数.
不管怎样, 自旋是很真实的, 而且是日常物理学中的重要组分.

整数自旋粒子 -- 被称为玻色子 -- 包括光子.
它无静止质量, 某种程度上是特殊的, 只有自旋 ±1 的组分 (而没有零自旋),
对应于两个熟知的偏振正交态.

半整数自旋粒子, 被称为费米子, 包括电子, 质子和中子.
这些是 "自旋 1/2" 粒子, 它们的自旋分量要么是 +1/2 (自旋 "向上"),
要么是 -1/2 (自旋 "向下"). 当我们提到 "自旋", 通常是指自旋 1/2 粒子.

诸如位置和动量的连续变量, 以及其他无限的希尔伯特空间系统, 必须被人为地截断以便代表量子比特.
与它们不同, 自旋态为量子信息提供了一个良好的表示, 因为它们天然地存在有限的态空间.
```

- [声子](https://en.wikipedia.org/wiki/Phonon)

## 附录

### B. 群论

- [隐藏子群问题](https://en.wikipedia.org/wiki/Hidden_subgroup_problem)

- 元素
  $$ g \in G $$
  的阶是使得
  $$ g^r $$
  (`g` 自乘 `r` 次) 等于单位元 `e` 的最小正整数.
- 若 `H` 是 `G` 的子集, 且与 `G` 在相同乘法运算下构成群, 则称 `H` 是群 `G` 的子群.
- __拉格朗日定理__ 如果 `H` 是一个有限群 `G` 的子群, 那么 `|H|` 可以整除 `|G|`.

- 如果
  $$ g_1 $$
  和
  $$ g_2 $$
  是 `G` 中的元素, 那么
  $$ g_2 $$
  关于
  $$ g_1 $$
  的共轭为元素
  $$ g_1^{-1} g_2 g_1 $$.
  - 若 `H` 是 `G` 的子群, 且
    $$ g^{-1} H g = H $$
    对所有
    $$ g \in G $$
    成立, 则称其为`正规子群`.
  - 群 `G` 中的元素 `x` 的`共轭类`
    $$ G_x $$,
    定义为
    $$ G_x ≡ \{ g^{-1} x g | g \in G \} $$.

- [生成元](https://en.wikipedia.org/wiki/Generating_set_of_a_group)

- 循环群 `G` 包含一个元素 `a`, 使得任意元素
  $$ g \in G $$
  能够表示为
  $$ a^n $$,
  其中 `n` 为某个整数.
  - `a` 称为 `G` 的`生成元`, 我们记
    $$ G = \langle a \rangle $$.
  - 由
    $$ g \in G $$
    生成的`循环子群` `H` 是指由
    $$ \{ e, g, g^2, ..., g^{r-1} \} $$
    构成的群, 其中 `r` 为 `g` 的阶. 即
    $$ H = \langle H \rangle $$.

- 若 `H` 是 `G` 的一个子群,
  由 `g` 所确定的 `H` 在 `G` 中的`左陪集`定义为集合
  $$ gH ≡ \{ gh | h \in H \} $$.
  - 右陪集定义类似.
  - 陪集是左陪集还是右陪集可以从上下文看出.
  - 对像
    $$ \mathbb{Z}_n $$
    的群运算为加法的群, 习惯上把子群 `H` 对
    $$ g \in \mathbb{Z}_n $$
    的陪集写成
    $$ g + H $$
    的形式.
  - 陪集
    $$ gH $$
    的一个特定的元素被称为该陪集的代表元.

---

- 令
  $$ M_n $$
  表示
  $$ n \times n $$
  复矩阵的集合. 那么一个矩阵群是
  $$ M_n $$
  的集合, 它在矩阵乘法下满足群的性质.
  我们记这类群的单位元为 `I`. 一个群 `G` 的表示
  $$ ρ $$
  定义为一个函数, 该函数将 `G` 映射到一个矩阵群, 且保持群的乘法运算.
  - 特别地,
    $$ g \in G $$
    被映射到
    $$ ρ(g) \in M_n $$,
    使得
    $$ g_1 g_2 = g_3 $$
    蕴含
    $$ ρ(g_1) ρ(g_2) = ρ(g_3) $$.
  - 如果映射是多对一的, 则称之为`同态`;
  - 如果是一对一的, 则称之为`同构`.
- 一个映射到
  $$ M_n $$
  的表示
  $$ ρ $$
  具有维数
  $$ d_n = n $$.
  我们定义的表示也称为矩阵表示, 还有更一般的表示,
  但是对我们来说矩阵表示足够用了.

- [矩阵群](https://en.wikipedia.org/wiki/Linear_group)
  - [表示](https://en.wikipedia.org/wiki/Group_representation)
  - [特征](https://en.wikipedia.org/wiki/Character_theory)

- [有限群上的傅立叶变换](https://en.wikipedia.org/wiki/Fourier_transform_on_finite_groups)

> 最后 `4` 页的信息量蛮大~

### D. 数论
